% dalsqgl - DAL with squared loss and grouped L1 regularization
%
% Overview:
%  Solves the optimization problem:
%   xx = argmin ||A*x-bb||^2 + lambda*||x||_G1
%  where
%   ||x||_G1 = sum(sqrt(sum(xx.^2)))
%  (grouped L1 norm)
%
% Syntax:
%  [xx,status]=dalsqgl(xx, A, bb, lambda, <opt>)
%
% Inputs:
%  xx     : initial solution ([ns,nc])
%  A      : the design matrix A ([mm,nn] or [mm,ns,nc]) or a cell
%           array {fA, fAT, mm, nn} where fA and fAT are function
%           handles to the functions that return A(:,:)*x and 
%           A(:,:)'*y, respectively, and mm and nn (=ns*nc) are the
%           numbers of rows and columns of A.
%           NOTE: in the definition of fA, x must be a matrix with
%           nn rows, i.e., fA=@(x)A*x; not fA=@(x)A*x(:)!
%  bb     : the target vector ([mm,1])
%  lambda : the regularization constant
%  <opt>  : list of 'fieldname1', value1, 'filedname2', value2, ...
%   stopcond : stopping condition, which can be
%              'pdg'  : Use relative primal dual gap (default)
%              'fval' : Use the objective function value
%           (see dal.m for other options)
% Outputs:
%  xx     : the final solution ([ns,nc])
%  status : various status values
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt

function [xx,status]=dalsqgl(xx, A, bb, lambda, varargin)

opt=propertylist2struct(varargin{:});
opt=set_defaults(opt,'solver','nt',...
                     'stopcond','pdg');

prob.floss    = struct('p',@loss_sqp,'d',@loss_sqd,'args',{{bb}});
prob.freg     = @(xx)sum(sqrt(sum(xx.^2)));
prob.obj      = @objdalgl;
prob.softth   = @gl_softth;
prob.stopcond = str2func(['gl_stopcond_' opt.stopcond]);

if isequal(opt.solver,'cg')
  prob.hessMult = @hessMultdalgl;
end

if isequal(opt.stopcond,'fval')
  opt.feval = 1;
end

if isnumeric(A)
  A = A(:,:);
  [mm,nn]=size(A);
  At=A';
  fAT = @(x)At*x;
  fA  = @(x)A*x;
  clear At;
elseif iscell(A)
  fA  = A{1};
  fAT = A{2};
  mm = A{3};
  nn = A{4};
else
  error('A must be either numeric or cell {@(x)A*x, @(y)(A''*y), mm, nn}');
end

prob.mm       = mm;
prob.nn       = nn;

[xx,status]=dal(prob,xx,fA,fAT,lambda,opt);



