% dalsql1 - DAL with squared loss and L1 regularization
%
% Overview:
%  Solves the optimization problem:
%   xx = argmin ||A*x-bb||^2 + lambda*||x||_1
%
% Syntax:
%  [xx,status]=dalsql1(xx, A, bb, lambda, <opt>)
%
% Inputs:
%  xx     : initial solution ([nn,1])
%  A      : the design matrix A ([mm,nn]) or a cell array {fA, fAT, mm, nn}
%           where fA and fAT are function handles to the functions that
%           return A*x and A'*x, respectively, and mm and nn are the
%           numbers of rows and columns of A.
%  bb     : the target vector ([mm,1])
%  lambda : the regularization constant
%  <opt>  : list of 'fieldname1', value1, 'filedname2', value2, ...
%   stopcond : stopping condition, which can be
%              'pdg'  : Use relative primal dual gap (default)
%              'fval' : Use the objective function value
%           (see dal.m for other options)
% Outputs:
%  xx     : the final solution ([nn,1])
%  status : various status values
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt

function [xx,status]=dalsql1(xx, A, bb, lambda, varargin)

opt=propertylist2struct(varargin{:});
opt=set_defaults(opt,'solver','nt',...
                     'stopcond','pdg');

prob.floss    = struct('p',@loss_sqp,'d',@loss_sqd,'args',{{bb}});
prob.freg     = @(xx)sum(abs(xx));
prob.obj      = @objdall1;
prob.softth   = @l1_softth;
prob.stopcond = str2func(['l1_stopcond_' opt.stopcond]);

if isequal(opt.solver,'cg')
  prob.hessMult = @hessMultdall1;
end

if isequal(opt.stopcond,'fval')
  opt.feval = 1;
end

if isnumeric(A)
  A = A(:,:);
  [mm,nn]=size(A);
  At=A';
  fAT = @(x)At*x;
  fA  = @(x)A*x;
  clear At;
elseif iscell(A)
  fA  = A{1};
  fAT = A{2};
  mm = A{3};
  nn = A{4};
else
  error('A must be either numeric or cell {@(x)A*x, @(y)(A''*y), mm, nn}');
end

prob.mm       = mm;
prob.nn       = nn;

[xx,status]=dal(prob,xx,fA,fAT,lambda,opt);

