% newton - a simple implementation of the Newton method
%
% Syntax:
%  [xx,fval,gg,status]=newton(fun, xx, tol, verbose);
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt
%
function [xx,fval,gg,status]=newton(fun, xx, tol, verbose);

if isempty(verbose)
  verbose=0;
end

if iscell(fun)
  fcnHessMult = fun{2};
  fun = fun{1};
else
  fcnHessMult = [];
end

n = size(xx,1);
if verbose
  fprintf('n=%d tol=%g\n', n, tol);
end


cc = 0;
step = nan;
while 1
  if isempty(fcnHessMult)
    [fval,gg,H]=feval(fun, xx);
  else
    [fval,gg,Hinfo]=feval(fun, xx);
  end
  
  if verbose
    fprintf('[%d] fval=%g norm(gg)=%g step=%g\n',cc, fval, norm(gg),step);
  end

  if norm(gg)<tol
    %fprintf('Optimization success!\n');
    ret = 0;
    status=archive('ret','cc');
    break;
  end

  if isempty(fcnHessMult)
    R = chol(H);
    dd = -R\(R'\gg);
  else
    [dd,dum1,dum2] = pcg(@(xx)fcnHessMult(xx,Hinfo), gg, max(1e-6,tol*0.01), length(xx),Hinfo.prec);
    dd = -dd;
  end
  
  [step,fval]=linesearch(fun, fval, xx, dd);

  if step==0
    ret = -2;
    fprintf('[newton] max linesearch reached.\n');
    status=archive('ret','cc');
    break;
  end
  
  xx = xx + step*dd;
  
  cc = cc+1;
  if cc>1000
    ret = -1;
    fprintf('[newton] #iterations>1000.\n');
    status=archive('ret','cc');
    break;
  end
end


function [step,fval] = linesearch(fun, fval0, xx, dd);

step = 1.0;

xx0    = xx;

cc = 1;
while 1
  xx = xx0 + step*dd;
  fval = feval(fun, xx);

  if fval<fval0
    break;
  end
  %fprintf('step=%g fval=%g (fval0=%g)\n',step, fval, fval0);
  
  step = step/2;
  cc = cc+1;
  if cc>30
    step = 0;
    break;
  end
end
